package com.fzm.common.config;

import org.springframework.amqp.core.*;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @author tangtuo
 * @date 2021/12/23 13:25
 */
@Configuration
public class RabbitMQConfig {

    public static final String COPYRIGHT_DIRECT = "copyright.direct";

    public static final String DEAD_LETTER_DIRECT = "dl.direct";

    public static final String COPYRIGHT_QUEUE = "copyright.queue";

    public static final String DEAD_LETTER_QUEUE = "dl.queue";


    public static final String EVIDENCE_HASH_QUEUE = "evidence.hash.queue";

    public static final String EVIDENCE_DEAD_LETTER_QUEUE = "evidence.dl.queue";


    /**
     * 定义直连交换机copyright.direct
     *
     * @return
     */
    @Bean
    public DirectExchange copyrightDirect() {
        return new DirectExchange(COPYRIGHT_DIRECT, true, false, null);
    }

    /**
     * 死信交换机
     *
     * @return
     */
    @Bean
    public DirectExchange dlDirect() {
        return new DirectExchange(DEAD_LETTER_DIRECT, true, false, null);
    }

    /**
     * 存放死信的队列,用户监听这个队列
     *
     * @return
     */
    @Bean
    public Queue copyrightQueue() {
        return new Queue(COPYRIGHT_QUEUE, true);
    }


    /**
     * 定义死信队列
     *
     * @return
     */
    @Bean
    public Queue dlQueue() {
        return QueueBuilder.durable(DEAD_LETTER_QUEUE)
                .ttl(1000 * 60 * 60 * 6)
                .ttl(1000 * 60 * 2) // 测试环境
                .deadLetterExchange(DEAD_LETTER_DIRECT)
                .deadLetterRoutingKey("copyright.notify")
                .build();
    }

    @Bean
    public Binding dlBinding() {
        return BindingBuilder.bind(dlQueue()).to(copyrightDirect()).with("copyright.apply");
    }

    @Bean
    public Binding copyrightBinding() {
        return BindingBuilder.bind(copyrightQueue()).to(dlDirect()).with("copyright.notify");
    }


    /**
     * 用户追加tokenInfo时,通过evmSetTokenInfo()方法得到交易hash,然后通过getRealTxHashFromGrp()得到真实哈希,
     * 但是需要阻塞3秒左右才能查到真实哈希,所以这里使用死信加延时队列
     * <p>
     * 存放交易哈希的队列,用户监听这个队列
     *
     * @return
     */
    @Bean
    public Queue evidenceHashQueue() {
        return new Queue(EVIDENCE_HASH_QUEUE, true);
    }


    /**
     * 定义存放交易哈希的死信队列,这个队列没有监听者,消息过期后会路由到 EVIDENCE_HASH_QUEUE 这个队列
     *
     * @return
     */
    @Bean
    public Queue evidenceDlQueue() {
        return QueueBuilder.durable(EVIDENCE_DEAD_LETTER_QUEUE)
                .ttl(1000 * 10)
                .deadLetterExchange(DEAD_LETTER_DIRECT)
                .deadLetterRoutingKey("evidence.hash.query")
                .build();
    }

    @Bean
    public Binding evidenceHashBinding() {
        return BindingBuilder.bind(evidenceDlQueue()).to(copyrightDirect()).with("evidence.hash.query");
    }

    @Bean
    public Binding evidenceDlBinding() {
        return BindingBuilder.bind(evidenceHashQueue()).to(dlDirect()).with("evidence.hash.query");
    }

    @Bean
    public MessageConverter messageConverter() {
        return new Jackson2JsonMessageConverter();
    }
}
