package com.fzm.common.interceptor;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ArrayUtil;
import com.fzm.common.annotation.Authentication;
import com.fzm.common.constant.RedisConstant;
import com.fzm.common.constant.TokenConstant;
import com.fzm.common.entity.AbstractUser;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.service.AdminService;
import com.fzm.common.service.UserService;
import com.fzm.common.utils.JwtUtil;
import com.fzm.common.utils.RedisUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;
import java.util.Date;

/**
 * @author tangtuo
 * @date 2021/7/13 14:30
 */
public class AuthenticationInterceptor implements HandlerInterceptor {

    @Resource
    private RedisUtil redisUtil;

    @Resource
    private UserService userService;

    @Resource
    private AdminService adminService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        // 如果不是映射到方法直接通过
        if (!(handler instanceof HandlerMethod)) {
            return true;
        }
        HandlerMethod handlerMethod = (HandlerMethod) handler;
        Method method = handlerMethod.getMethod();
        Class<?> cls = method.getDeclaringClass();
        // 先判断目标方法所在的类是否加了Authentication注解,再判断目标方法是否加了Authentication注解
        if (cls.isAnnotationPresent(Authentication.class) || method.isAnnotationPresent(Authentication.class)) {
            String token = request.getHeader("Authorization");
            if (StringUtils.isBlank(token)) {
                throw GlobalException.newException(ResultCode.UNAUTHORIZED, "请先登录");
            }
            if (!token.startsWith(TokenConstant.TOKEN_PREFIX)) {
                throw GlobalException.newException(ResultCode.UNAUTHORIZED, "非法token");
            }
            String realToken = token.substring(TokenConstant.TOKEN_PREFIX.length());
            if (StringUtils.isBlank(realToken) || "null".equals(realToken)) {
                throw GlobalException.newException(ResultCode.UNAUTHORIZED, "请先登录");
            }
            if (JwtUtil.isTokenExpired(token)) {
                throw GlobalException.newException(ResultCode.UNAUTHORIZED, "登录已过期");
            }
            String appId = JwtUtil.getAppIdFromToken(token);
            Integer userId = JwtUtil.getUserIdFromToken(token);
            AbstractUser user;
            String redisToken;
            if (TokenConstant.TOKEN_APP_ID_PORTAL.equals(appId)) {
                // 门户用户
                user = userService.getById(userId);
                redisToken = redisUtil.get(RedisConstant.PORTAL_USER_TOKEN_PREFIX + userId);
            } else {
                // 后台管理用户
                user = adminService.getById(userId);
                redisToken = redisUtil.get(RedisConstant.ADMIN_USER_TOKEN_PREFIX + userId);
            }
            if (user == null) {
                throw GlobalException.newException(ResultCode.UNAUTHORIZED, "非法用户请求");
            }
            if (StringUtils.isBlank(redisToken)) {
                throw GlobalException.newException(ResultCode.UNAUTHORIZED, "登录已过期");
            }
            if (!redisToken.equals(realToken)) {
                Date date = JwtUtil.getCreateTimeFromToken(realToken);
                String format = DateUtil.format(date, "yyyy-MM-dd HH:mm:ss");
                throw GlobalException.newException(ResultCode.UNAUTHORIZED,
                        String.format("您的账号于 %s 登录另外一台设备,如非您本人所为,请立即修改密码", format));
            }
            // todo 认证完毕,对后台用户进行鉴权
            /*if (TokenConstant.TOKEN_APP_ID_ADMIN.equals(appId)) {
                Authentication annotation = method.getAnnotation(Authentication.class);
                String[] permissions = annotation.permissions();
                if (ArrayUtil.isNotEmpty(permissions)) {

                }
            }*/

        }
        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) throws Exception {

    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) throws Exception {

    }

    public static void main(String[] args) {
        String token = "Bearer null ";
        System.out.println(token.substring("Bearer ".length()));
        System.out.println(StringUtils.isBlank(token.substring("Bearer ".length())));
    }
}
