package com.fzm.common.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.TypeReference;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fzm.common.constant.SystemConstant;
import com.fzm.common.entity.*;
import com.fzm.common.entity.dto.CopyrightDTO;
import com.fzm.common.entity.dto.CopyrightRequest;
import com.fzm.common.entity.dto.CopyrightResponse;
import com.fzm.common.entity.vo.CopyrightApplyVo;
import com.fzm.common.entity.vo.CopyrightCertificateVo;
import com.fzm.common.entity.vo.CopyrightVo;
import com.fzm.common.enums.CopyrightApplyState;
import com.fzm.common.enums.ResultCode;
import com.fzm.common.exception.GlobalException;
import com.fzm.common.mapper.CopyrightApplyMapper;
import com.fzm.common.properties.CopyrightProperties;
import com.fzm.common.service.*;
import com.fzm.common.utils.CopyrightSignUtil;
import com.fzm.common.utils.ObsUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.redisson.Redisson;
import org.redisson.api.RLock;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.TreeMap;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;

/**
 * @author tangtuo
 * @date 2021/12/9 11:40
 */
@Slf4j
@Service
@Transactional(rollbackFor = RuntimeException.class)
public class CopyrightApplyServiceImpl extends ServiceImpl<CopyrightApplyMapper, CopyrightApply> implements CopyrightApplyService {

    @Resource
    private CopyrightApplyMapper copyrightApplyMapper;

    @Resource
    private CopyrightFileService copyrightFileService;

    @Resource
    private CopyrightApplyOwnerRelationService copyrightApplyOwnerRelationService;

    @Resource
    private CopyrightAuthorService copyrightAuthorService;

    @Resource
    private CopyrightAuthorityRelationService copyrightAuthorityRelationService;

    @Resource
    private AuthorityService authorityService;

    @Resource
    private CopyrightOwnerService copyrightOwnerService;

    @Resource
    private UserService userService;

    @Resource
    private NftService nftService;

    @Resource
    private ObsUtil obsUtil;

    @Resource
    private ThreadPoolTaskExecutor threadPoolTaskExecutor;

    @Resource
    private OpusCategoryService opusCategoryService;

    @Resource
    private CopyrightProperties copyrightProperties;

    @Resource
    private Redisson redisson;

    @Resource
    private DraftService draftService;


    @Override
    public Integer submit(CopyrightDTO copyrightDTO) {
        // 判断此nft是否属于当前登录用户所有
        Nft nft = nftService.getByNftHash(copyrightDTO.getNftHash());
        User user = userService.getUserByToken();
        if (nft == null || !nft.getUserId().equals(user.getId())) {
            throw GlobalException.newException(ResultCode.FAILED, "请先确认此nft的有效性");
        }
        // 如果发表状态是已发表的话,需要传入首次发表日期和省市
        if (SystemConstant.BOOLEAN_DATA_TRUE.equals(copyrightDTO.getPublishState()) &&
                (copyrightDTO.getFirstPublishDate() == null ||
                        StringUtils.isAnyBlank(copyrightDTO.getFirstPublishCity(), copyrightDTO.getFirstPublishProvince()))) {
            throw GlobalException.newException(ResultCode.VALIDATE_FAILED);
        }
        CopyrightApply copyrightApply = new CopyrightApply(copyrightDTO);
        // 首次提交,登记状态为待提交
        copyrightApply.setRegisterState(CopyrightApplyState.TO_BE_REVIEWED.getCode());
        copyrightApply.setUserId(user.getId());
        copyrightApply.setApplyTime(new Date());
        try {
            save(copyrightApply);
        } catch (DuplicateKeyException e) {
            throw GlobalException.newException(ResultCode.FAILED, "当前存证已提交过版权申请, 请勿重复提交");
        }

        // 保存附件
        ArrayList<CopyrightFile> files = new ArrayList<>();
        for (CopyrightDTO.File file : copyrightDTO.getFiles()) {
            CopyrightFile copyrightFile = new CopyrightFile(file);
            copyrightFile.setNftHash(copyrightApply.getNftHash());
            copyrightFile.setCopyrightId(copyrightApply.getId());
            files.add(copyrightFile);
        }
        copyrightFileService.saveBatch(files);
        // 保存著作权人信息
        ArrayList<CopyrightApplyOwnerRelation> owners = new ArrayList<>();
        for (Integer ownerId : copyrightDTO.getOwnerIds()) {
            owners.add(new CopyrightApplyOwnerRelation(copyrightApply.getId(), copyrightOwnerService.getById(ownerId)));
        }
        copyrightApplyOwnerRelationService.saveBatch(owners);
        // 保存作者信息
        ArrayList<CopyrightAuthor> authors = new ArrayList<>();
        for (CopyrightDTO.Author author : copyrightDTO.getAuthors()) {
            CopyrightAuthor copyrightAuthor = new CopyrightAuthor(author);
            copyrightAuthor.setCopyrightId(copyrightApply.getId());
            authors.add(copyrightAuthor);
        }
        copyrightAuthorService.saveBatch(authors);
        // 保存权力信息
        ArrayList<CopyrightAuthorityRelation> relations = new ArrayList<>();
        for (Integer authorityId : copyrightDTO.getAuthorityIds()) {
            CopyrightAuthorityRelation relation = new CopyrightAuthorityRelation(copyrightApply.getId(), authorityId);
            relations.add(relation);
        }
        // 如果是从草稿箱进来的需要删除草稿信息
        if (copyrightDTO.getDraftId() != null) {
            draftService.removeById(copyrightDTO.getDraftId());
        }
        copyrightAuthorityRelationService.saveBatch(relations);
        return copyrightApply.getId();
    }

    @Override
    public List<CopyrightApply> list(Integer userId, Integer state) {
        return copyrightApplyMapper.list(userId, state);
    }

    @Override
    public Boolean withdraw(Integer id) {
        CopyrightApply copyright = getById(id);
        if (copyright == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "此版权登记记录不存在,请核对后重试");
        }
        if (CopyrightApplyState.TO_BE_REVIEWED.getCode() != copyright.getRegisterState()) {
            throw GlobalException.newException(ResultCode.OPERATION_FAILED, "此版权申请记录已提交,无法撤回");
        }
        copyright.setRegisterState(CopyrightApplyState.WITHDRAW.getCode());
        return updateById(copyright);
    }


    @Override
    public Boolean delete(Integer id) {
        CopyrightApply copyright = getById(id);
        if (copyright == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "此版权登记记录不存在,请核对后重试");
        }
        // 只有撤回、驳回和登记失败的才能删除
        if (CopyrightApplyState.WITHDRAW.getCode() != copyright.getRegisterState() &&
                CopyrightApplyState.REJECTED.getCode() != copyright.getRegisterState() &&
                CopyrightApplyState.FAILED.getCode() != copyright.getRegisterState()) {
            throw GlobalException.newException(ResultCode.OPERATION_FAILED, "当前版权记录不能被删除");
        }
        // 删除权力取得方式的附件
        if (StringUtils.isNotBlank(copyright.getAuthorityAcquireProve())) {
            obsUtil.delete(copyright.getAuthorityAcquireProve());
        }
        // 删除权力归属证明附件
        if (StringUtils.isNotBlank(copyright.getAuthorityAscriptionProve())) {
            obsUtil.delete(copyright.getAuthorityAscriptionProve());
        }
        // 删除附件列表
        copyrightFileService.delByCopyrightId(id);
        // 删除作者列表
        copyrightAuthorService.delByCopyrightId(id);
        // 删除版权权力拥有情况
        copyrightAuthorityRelationService.delByCopyrightId(id);
        // 删除版权和著作权人关联关系
        copyrightApplyOwnerRelationService.delByCopyrightId(id);
        return copyrightApplyMapper.deleteById(id) > 0;
    }

    @Override
    public CopyrightVo getDetail(Integer id) throws ExecutionException, InterruptedException {
        // 使用异步编排查询
        CompletableFuture<CopyrightVo> applyFuture = CompletableFuture.supplyAsync(() -> {
            CopyrightApply copyrightApply = getById(id);
            return new CopyrightVo(copyrightApply);
        }, threadPoolTaskExecutor);
        CompletableFuture<Void> categoryFuture = applyFuture.thenAcceptAsync((result) -> {
            // 查询类别
            OpusCategory category = opusCategoryService.getById(result.getOpusCategoryId());
            result.setOpusCategory(category);
        }, threadPoolTaskExecutor);
        CompletableFuture<Void> filesFuture = applyFuture.thenAcceptAsync((result) -> {
            // 查询附件列表
            List<CopyrightFile> files = copyrightFileService.getByCopyrightId(id);
            result.setFiles(files);
        }, threadPoolTaskExecutor);
        CompletableFuture<Void> authorsFuture = applyFuture.thenAcceptAsync((result) -> {
            // 查询作者列表
            List<CopyrightAuthor> authors = copyrightAuthorService.getByCopyrightId(id);
            result.setAuthors(authors);
        }, threadPoolTaskExecutor);
        CompletableFuture<Void> ownerFuture = applyFuture.thenAcceptAsync((result) -> {
            // 查询著作权人列表
            List<CopyrightApplyOwnerRelation> owners = copyrightApplyOwnerRelationService.getByCopyrightId(id);
            result.setOwners(owners);
        }, threadPoolTaskExecutor);
        CompletableFuture<Void> authoritiesFuture = applyFuture.thenAcceptAsync((result) -> {
            // 查询权力列表
            List<Authority> authorities = authorityService.getByCopyrightId(id);
            result.setAuthorities(authorities);
        }, threadPoolTaskExecutor);
        // 等所有异步任务都完成以后返回结果
        CompletableFuture.allOf(categoryFuture, filesFuture, authorsFuture, ownerFuture, authoritiesFuture).get();
        return applyFuture.get();
    }

    @Override
    public Boolean update(CopyrightDTO copyrightDTO) {
        // 只有撤回 驳回和登记失败的才能编辑
        CopyrightApply copyright = this.getById(copyrightDTO.getId());
        if (CopyrightApplyState.REJECTED.getCode() != copyright.getRegisterState() &&
                CopyrightApplyState.WITHDRAW.getCode() != copyright.getRegisterState() &&
                CopyrightApplyState.FAILED.getCode() != copyright.getRegisterState()) {
            throw GlobalException.newException(ResultCode.OPERATION_FAILED, "只有撤销、驳回或登记失败的版权记录才能编辑");
        }
        CopyrightApply copyrightApply = new CopyrightApply(copyrightDTO);
        if (CollectionUtil.isNotEmpty(copyrightDTO.getFiles())) {
            copyrightFileService.delByCopyrightId(copyrightDTO.getId());
            ArrayList<CopyrightFile> files = new ArrayList<>();
            for (CopyrightDTO.File file : copyrightDTO.getFiles()) {
                CopyrightFile copyrightFile = new CopyrightFile(file);
                copyrightFile.setNftHash(copyrightApply.getNftHash());
                copyrightFile.setCopyrightId(copyrightApply.getId());
                files.add(copyrightFile);
            }
            copyrightFileService.saveBatch(files);
        }
        if (CollectionUtil.isNotEmpty(copyrightDTO.getAuthors())) {
            // 删除作者信息
            copyrightAuthorService.delByCopyrightId(copyrightDTO.getId());
            // 保存作者信息
            ArrayList<CopyrightAuthor> authors = new ArrayList<>();
            for (CopyrightDTO.Author author : copyrightDTO.getAuthors()) {
                CopyrightAuthor copyrightAuthor = new CopyrightAuthor(author);
                copyrightAuthor.setCopyrightId(copyrightApply.getId());
                authors.add(copyrightAuthor);
            }
            copyrightAuthorService.saveBatch(authors);
        }
        if (CollectionUtil.isNotEmpty(copyrightDTO.getAuthorityIds())) {
            // 删除权力信息
            copyrightAuthorityRelationService.delByCopyrightId(copyrightDTO.getId());
            // 保存权力信息
            ArrayList<CopyrightAuthorityRelation> relations = new ArrayList<>();
            for (Integer authorityId : copyrightDTO.getAuthorityIds()) {
                CopyrightAuthorityRelation relation = new CopyrightAuthorityRelation(copyrightApply.getId(), authorityId);
                relations.add(relation);
            }
            copyrightAuthorityRelationService.saveBatch(relations);
        }
        if (CollectionUtil.isNotEmpty(copyrightDTO.getOwnerIds())) {
            // 删除著作权人信息
            copyrightApplyOwnerRelationService.delByCopyrightId(copyrightDTO.getId());
            // 保存著作权人信息
            ArrayList<CopyrightApplyOwnerRelation> owners = new ArrayList<>();
            for (Integer ownerId : copyrightDTO.getOwnerIds()) {
                owners.add(new CopyrightApplyOwnerRelation(copyrightApply.getId(), copyrightOwnerService.getById(ownerId)));
            }
            copyrightApplyOwnerRelationService.saveBatch(owners);
        }
        copyrightApply.setApplyTime(new Date());
        copyrightApply.setRegisterState(CopyrightApplyState.TO_BE_REVIEWED.getCode());
        return updateById(copyrightApply);
    }

    @Override
    public PageInfo<CopyrightApplyVo> pages(Integer pageNum, Integer pageSize, String telephone, String serialNum, Integer registerState, String from, String to) {
        PageHelper.startPage(pageNum, pageSize);
        DateTime startDate = null;
        DateTime endDate = null;
        if (StringUtils.isNotBlank(from)) {
            startDate = DateUtil.parse(from + " 00:00:00", "yyyy-MM-dd HH:mm:ss");
        }
        if (StringUtils.isNotBlank(to)) {
            endDate = DateUtil.parse(to + " 23:59:59", "yyyy-MM-dd HH:mm:ss");
        }
        List<CopyrightApplyVo> list = copyrightApplyMapper.pages(telephone, serialNum, registerState, startDate, endDate);
        return new PageInfo<>(list);
    }

    @Override
    public boolean reject(Integer id, String rejectReason) {
        CopyrightApply copyright = getById(id);
        if (copyright == null) {
            throw GlobalException.newException(ResultCode.DATA_ERROR, "此版权登记记录不存在,请核对后重试");
        }
        if (CopyrightApplyState.TO_BE_REVIEWED.getCode() != copyright.getRegisterState()) {
            throw GlobalException.newException(ResultCode.OPERATION_FAILED, "此版权申请记录无法驳回");
        }
        copyright.setRegisterState(CopyrightApplyState.REJECTED.getCode());
        copyright.setRejectReason(rejectReason);
        return updateById(copyright);
    }

    @Override
    public boolean pass(Integer id) throws ExecutionException, InterruptedException {
        RLock lock = redisson.getLock("copyright:apply:" + id);
        boolean tryLock = lock.tryLock(10, TimeUnit.SECONDS);
        if (!tryLock) {
            throw GlobalException.newException(ResultCode.FAILED, "操作频繁");
        }
        try {
            CopyrightApply copyright = getById(id);
            if (copyright == null) {
                throw GlobalException.newException(ResultCode.DATA_ERROR, "此版权登记记录不存在,请核对后重试");
            }
            if (CopyrightApplyState.TO_BE_REVIEWED.getCode() != copyright.getRegisterState()) {
                throw GlobalException.newException(ResultCode.OPERATION_FAILED, "此版权申请记录的状态不是待提交,无法审核通过");
            }
            // 请求版权局接口
            CopyrightVo detail = this.getDetail(id);
            OpusCategory category = opusCategoryService.getById(detail.getOpusCategoryId());
            CopyrightRequest copyrightRequest = new CopyrightRequest(detail, category, copyrightProperties.getAppId(), null);
            TreeMap<String, String> param = this.beanToMap(copyrightRequest, copyrightProperties.getAppSecret());
            String sign = CopyrightSignUtil.generateSign(param);
            copyrightRequest.setSign(sign);
            String json = JSONUtil.toJsonStr(copyrightRequest);
            log.info("请求版权局接口, 接口地址:{}, 请求参数:{}", copyrightProperties.getApplyUrl(), json);
            String response = HttpUtil.post(copyrightProperties.getApplyUrl(), json, 10000);
            log.info("版权局响应内容:{}", response);
            if (StringUtils.isBlank(response)) {
                throw GlobalException.newException(ResultCode.COPYRIGHT_FAILED, "版权局接口无响应,请稍后重试");
            }
            CopyrightResponse copyrightResponse = JSONUtil.toBean(response, CopyrightResponse.class);
            if (copyrightResponse == null) {
                throw GlobalException.newException(ResultCode.COPYRIGHT_FAILED, "版权局接口无响应,请稍后重试");
            }
            if (!copyrightResponse.isSuccess()) {
                throw GlobalException.newException(ResultCode.COPYRIGHT_FAILED, copyrightResponse.getMessage());
            }
            // 申请成功
            String serial_code = copyrightResponse.getSerial_code();
            copyright.setSerialNum(serial_code);
            copyright.setRegisterState(CopyrightApplyState.SUBMITTED.getCode());
            copyright.setRejectReason("");
            copyright.setPassTime(new Date());
            return updateById(copyright);
        } finally {
            lock.unlock();
        }

    }

    private TreeMap<String, String> beanToMap(CopyrightRequest copyrightRequest, String app_secret) {
        TreeMap<String, String> treeMap = JSONUtil.toBean(JSONUtil.toJsonStr(copyrightRequest), new TypeReference<TreeMap<String, String>>() {
        }, true);
        treeMap.put("app_secret", app_secret);
        return treeMap;

    }

    @Override
    public int getSuccessCount() {
        QueryWrapper<CopyrightApply> wrapper = new QueryWrapper<>();
        wrapper.eq("register_state", CopyrightApplyState.SUCCEEDED.getCode());
        return this.count(wrapper);
    }

    @Override
    public CopyrightApply getBySerialNum(String serial_code) {
        QueryWrapper<CopyrightApply> wrapper = new QueryWrapper<>();
        wrapper.eq("serial_num", serial_code);
        return getOne(wrapper);
    }

    @Override
    public CopyrightVo getByNftHash(String nftHash) throws ExecutionException, InterruptedException {
        QueryWrapper<CopyrightApply> wrapper = new QueryWrapper<>();
        wrapper.eq("nft_hash", nftHash);
        CopyrightApply copyrightApply = this.getOne(wrapper);
        if (copyrightApply == null) {
            return null;
        }
        return this.getDetail(copyrightApply.getId());
    }

    @Override
    public CopyrightCertificateVo getCertificate(Integer id) {
        CopyrightApply copyrightApply = this.getById(id);
        if (copyrightApply == null) {
            throw GlobalException.newException(ResultCode.SELECT_FAILED, "没找到此证书");
        }
        if (!copyrightApply.getRegisterState().equals(CopyrightApplyState.SUCCEEDED.getCode())) {
            throw GlobalException.newException(ResultCode.FAILED, "当前版权申请还未审批通过,暂不支持查看证书");
        }
        OpusCategory category = opusCategoryService.getById(copyrightApply.getOpusCategoryId());
        User user = userService.getById(copyrightApply.getUserId());
        return new CopyrightCertificateVo(copyrightApply, user.getNickname(), category.getValue());
    }

    @Override
    public Boolean existsByNftHash(String nftHash) {
        QueryWrapper<CopyrightApply> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("nft_hash", nftHash);
        return count(queryWrapper) > 0;
    }

    @Override
    public List<String> getSerialCodes() {
        return copyrightApplyMapper.getSerialCodes();
    }

}
