package com.fzm.portal.aop;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.After;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.lang.reflect.Parameter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author tangtuo
 * @date 2021/9/10 14:27
 */
@Slf4j
@Order(1)
@Component
@Aspect
public class LogAop {

    @Resource
    private HttpServletRequest request;

    private static ThreadLocal<Long> threadLocal = new ThreadLocal<>();

    @Pointcut(value = "execution(public * com.fzm.portal.controller..*.*(..))")
    public void pointCut() {
    }


    /**
     * 前置通知
     *
     * @param joinPoint
     */
    @Before("pointCut()")
    public void deBefore(JoinPoint joinPoint) {
        // 记录此次请求的开始时间
        long start = System.currentTimeMillis();
        threadLocal.set(start);
        // 获取请求路径
        String requestURI = request.getRequestURI();
        // 获取请求参数
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        Method method = signature.getMethod();
        List<Object> parameter = getParameter(method, joinPoint.getArgs());
        log.info("请求路径: {}，请求参数：{}", requestURI, parameter);
    }


    /**
     * 后置通知
     *
     * @param joinPoint
     */
    @After("pointCut()")
    public void doAfter(JoinPoint joinPoint) {
        long end = System.currentTimeMillis();
        Long start = threadLocal.get();
        long l = end - start;
        String requestURI = request.getRequestURI();
        log.info("请求路径：{}，接口耗时：{}ms", requestURI, l);
    }

    /**
     * 根据方法和传入的参数获取请求参数
     */
    private List<Object> getParameter(Method method, Object[] args) {
        List<Object> argList = new ArrayList<>();
        Parameter[] parameters = method.getParameters();
        for (int i = 0; i < parameters.length; i++) {
            //将RequestBody注解修饰的参数作为请求参数
            RequestBody requestBody = parameters[i].getAnnotation(RequestBody.class);
            if (requestBody != null) {
                argList.add(args[i]);
            }
            ModelAttribute modelAttribute = parameters[i].getAnnotation(ModelAttribute.class);
            if (modelAttribute != null) {
                argList.add(args[i]);
            }
            //将RequestParam注解修饰的参数作为请求参数
            RequestParam requestParam = parameters[i].getAnnotation(RequestParam.class);
            if (requestParam != null) {
                Map<String, Object> map = new HashMap<>();
                String key = parameters[i].getName();
                if (!StringUtils.isEmpty(requestParam.value())) {
                    key = requestParam.value();
                }
                map.put(key, args[i]);
                argList.add(map);
            }
            // PathVariable修饰的请求参数
            PathVariable pathVariable = parameters[i].getAnnotation(PathVariable.class);
            if (pathVariable != null) {
                Map<String, Object> map = new HashMap<>();
                String key = parameters[i].getName();
                if (StringUtils.isNotBlank(pathVariable.value())) {
                    key = pathVariable.value();
                }
                map.put(key, args[i]);
                argList.add(map);
            }
        }
        return argList;
    }

}
